<?php
/**
 * @package   ats
 * @copyright Copyright (c)2011-2020 Nicholas K. Dionysopoulos / Akeeba Ltd
 * @license   GNU General Public License version 3, or later
 */

namespace Akeeba\TicketSystem\Admin\Helper;

use Akeeba\TicketSystem\Admin\Model\Tickets;
use FOF30\Container\Container;
use FOF30\View\DataView\Raw;
use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Filesystem\Folder as JFolder;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Table\Table as JTable;
use Joomla\CMS\Uri\Uri as JUri;
use Joomla\CMS\User\User;

defined('_JEXEC') or die;

class Html
{
	/**
	 * Creates the CSS class accordingly to the ticket status
	 *
	 * @param $value
	 *
	 * @return string   CSS class for the ticket status
	 */
	public static function getStatusClass($value)
	{
		switch ($value)
		{
			case 'O':
				return 'akeeba-label--red ats-status-open';
			case 'C':
				return 'akeeba-label--green ats-status-close';
			case 'P':
				return 'akeeba-label--orange ats-status-pending';
			default:
				return 'akeeba-label--orange ats-status-' . $value;
		}
	}

	/**
	 * Create an HTML element accordingly to the post origin
	 *
	 * @param   string  $origin
	 *
	 * @return  string  Html element for the origin of the post
	 */
	public static function getPostOriginIcon($origin)
	{
		if ($origin == 'web')
		{
			$origin_icon = 'akion-earth ';
		}
		elseif ($origin == 'email')
		{
			$origin_icon = 'akion-email ';
		}
		else
		{
			$origin_icon = 'akion-ios-cog';
		}

		return $origin_icon;
	}

	/**
	 * Decode ticket status, including custom ones defined by the user
	 *
	 * @param   string|int  $status       Status to decode
	 * @param   boolean     $create_span  Should I wrap the output in a span?
	 *
	 * @return  string       Decoded HTML
	 * @deprecated
	 *
	 */
	public static function decodeStatus($status, $create_span = false)
	{
		$standard = ['O', 'P', 'C'];
		$custom   = Select::getCustomTicketStatuses();

		if (in_array($status, $standard))
		{
			$text  = Text::_('COM_ATS_TICKETS_STATUS_' . $status);
			$class = 'ats-ticket-status-' . $status;
		}
		elseif (isset($custom[$status]))
		{
			$text = $custom[$status];
			// Custom status are always considered "pending"
			$class = 'ats-ticket-status-P';
		}
		else
		{
			return '';
		}

		if ($create_span)
		{
			return '<span class="' . $class . '">' . $text . '</span>';
		}
		else
		{
			return $text;
		}
	}

	/**
	 * Uses the ATS plugins to fetch ana avatar for the user
	 *
	 * @param   User  $user  The user for which to fetch an avatar for
	 * @param   int   $size  The size (in pixels), defaults to 64
	 *
	 * @return string The URL to the avatar image
	 */
	public static function getAvatarURL($user, $size = 64)
	{
		$container = Container::getInstance('com_ats');
		$container->platform->importPlugin('ats');

		$jResponse = $container->platform->runPlugins('onATSAvatar', [$user, $size]);

		$url = '';

		if (!empty($jResponse))
		{
			foreach ($jResponse as $response)
			{
				if ($response)
				{
					$url = $response;
				}
			}
		}

		return $url;
	}

	/**
	 * Creates a dropdown list containing managers.
	 *
	 * @param   string   $selected  The **full name** of the assigned manager
	 * @param   Tickets  $ticket    Ticket category
	 *
	 * @return  string   Dropdown html code
	 */
	public static function getManagerAssignDropdown($selected, Tickets $ticket)
	{
		$container = Container::getInstance('com_ats');
		$category  = $ticket->catid;

		$myId     = $container->platform->getUser()->id;
		$managers = Permissions::getManagers($category);

		// Create options at the top of the managers list
		$fakeManagers = [];

		// -- Assign to me
		if (array_key_exists($myId, $managers))
		{
			$fakeManagers[] = (object) [
				'id'   => $myId,
				'name' => '<span class="akeeba-label--teal"><span class="akion-flag"></span></span> ' . Text::_('COM_ATS_TICKET_ASSIGN_TOME'),
			];
		}

		// -- Unassign
		$fakeManagers[] = (object) [
			'id'   => 0,
			'name' => '<span class="akeeba-label--red"><span class="akion-fireball"></span></span> ' . Text::_('COM_ATS_TICKET_UNASSIGN'),
		];

		$merged = array_merge($fakeManagers, $managers);

		$color      = 'dark';
		$me         = isset($managers[$myId]) ? $managers[$myId]->name : '';
		$unassigned = Text::_('COM_ATS_TICKETS_UNASSIGNED');

		switch ($selected)
		{
			case $unassigned:
				$color = 'grey';
				break;

			case $me:
				$color = 'teal';
				break;
		}

		$html = '<nav class="akeeba-dropdown">';
		$html .= '<button class="akeeba-btn--small--' . $color . '">' . $selected . '</button>';
		$html .= '<section>';


		foreach ($merged as $manager)
		{
			$link = '#';

			$html .= sprintf("<a href=\"%s\" data-userid=\"%s\" data-ticketid=\"%s\">%s</a>", $link, $manager->id, $ticket->getId(), $manager->name);
		}

		$html .= '</section>';
		$html .= '</nav>';

		return $html;
	}

	/**
	 * Returns an HTML dropdown list of statuses, including the custom ones created by the user
	 *
	 * @param   Tickets  $ticket  The ticket to create the drop-down list for
	 *
	 * @return   string   HTML for the dropdown list
	 */
	public static function getTicketStatusDropdown(Tickets $ticket)
	{
		$selected = $ticket->status;

		/**
		 * The statuses are arranged as Open, Pending, custom statuses, Closed. The reason is that internally the custom
		 * statuses are treated as Pending.
		 */
		$statuses = [
			'O' => Text::_('COM_ATS_TICKETS_STATUS_O'),
			'P' => Text::_('COM_ATS_TICKETS_STATUS_P'),
		];

		// Can't use array_merge since if custom status IDs are not sequential, they will end up messed up (ie 1,3,4 will become 1,2,3)
		foreach (Select::getCustomTicketStatuses() as $custom_id => $custom_status)
		{
			$statuses[$custom_id] = $custom_status;
		}

		$statuses['C'] = Text::_('COM_ATS_TICKETS_STATUS_C');

		$text = $statuses[$selected];

		switch ($selected)
		{
			case 'O':
				$status_class = 'red';
				break;
			case 'P':
				$status_class = 'orange';
				break;
			case 'C':
				$status_class = 'green';
				break;
			default:
				$status_class = 'grey';
				break;
		}

		$html = '<nav class="akeeba-dropdown">';
		$html .= '<button class="akeeba-btn--small--' . $status_class . '">' . $text . '</button>';
		$html .= '<section>';

		foreach ($statuses as $value => $text)
		{
			switch ($value)
			{
				case 'O':
					$status_class = 'red';
					break;
				case 'P':
					$status_class = 'orange';
					break;
				case 'C':
					$status_class = 'green';
					break;
				default:
					$status_class = 'grey';
					break;
			}

			$html .= sprintf(
				"<a href=\"#\" data-status=\"%s\" data-ticketid=\"%s\"><span class=\"akeeba-label--%s\">%s</span></a>",
				$value, $ticket->getId(), $status_class, $text
			);
		}

		$html .= '</section>';
		$html .= '</nav>';

		return $html;
	}

	public static function getSignature(User $user)
	{
		static $signatures = [];

		if ($user->username == 'system')
		{
			return '';
		}

		$container = Container::getInstance('com_ats');

		if (!array_key_exists($user->username, $signatures))
		{
			$db = $container->db;

			$query     = $db->getQuery(true)
				->select($db->qn('profile_value'))
				->from($db->qn('#__user_profiles'))
				->where($db->qn('user_id') . ' = ' . $db->q($user->id))
				->where($db->qn('profile_key') . ' = ' . $db->q('ats.signature'));
			$signature = $db->setQuery($query)->loadResult();

			if (!is_null($signature))
			{
				$signature = Filter::filterText($signature, $user);
			}

			/**
			 * If there is no linebreak, paragraph or table tag we assume a plain text signature, therefore in need for
			 * nl2br(). This is backwards compatibility to older versions of ATS.
			 */
			if ((stripos($signature, '<br') === false) && (stripos($signature, '< br') === false) && (stripos($signature, '</p') === false) && (stripos($signature, '<td') === false))
			{
				$signature = nl2br($signature);
			}

			$signatures[$user->username] = empty($signature) || is_null($signature) ? '' : $signature;
		}

		return $signatures[$user->username];
	}

	/**
	 * Fetches the correct user that posted the ticket
	 *
	 * @param   int  $userid
	 *
	 * @return User
	 */
	public static function getPostUser($userid)
	{
		$container = Container::getInstance('com_ats');

		if ($userid == -1)
		{
			$postUser = clone $container->platform->getUser();

			$postUser->username = 'system';
			$postUser->email    = 'noreply@example.com';
			$postUser->name     = Text::_('COM_ATS_CLI_SYSTEMUSERLABEL');
		}
		else
		{
			$postUser = $container->platform->getUser($userid);
		}

		return $postUser;
	}

	public static function loadposition($position, $style = -2)
	{
		return Container::getInstance('com_ats')->template->loadPosition($position, $style);
	}

	/**
	 * Gets the total amount of tickets created by a specific user
	 *
	 * @param   integer  $userid  ID of the user
	 *
	 * @return  integer Number of created tickets
	 */
	public static function getTicketsCount($userid)
	{
		static $count = [];

		if (!isset($count[$userid]))
		{
			$container = Container::getInstance('com_ats');

			// Let's directly use the db, so I can be faster
			$db = $container->db;

			$query          = $db->getQuery(true)
				->select('COUNT(*)')
				->from($db->qn('#__ats_tickets'))
				->where($db->qn('created_by') . ' = ' . $db->q($userid));
			$count[$userid] = $db->setQuery($query)->loadResult();
		}

		return $count[$userid];
	}

	/**
	 * Gets the total amount of time spent supporting a specific user
	 *
	 * @param   integer  $userid  ID of the user
	 *
	 * @return  integer  Total time spent supporting the user, in minutes
	 */
	public static function getTimeSpentPerUser($userid)
	{
		static $timePerUser = [];

		if (!isset($timePerUser[$userid]))
		{
			$container = Container::getInstance('com_ats');

			// Let's directly use the db, so I can be faster
			$db = $container->db;

			$query = $db->getQuery(true);
			$query
				->select('SUM(' . $query->qn('timespent') . ')')
				->from($db->qn('#__ats_tickets'))
				->where($db->qn('created_by') . ' = ' . $db->q($userid));
			$timePerUser[$userid] = $db->setQuery($query)->loadResult();
		}

		return $timePerUser[$userid];
	}

	/**
	 * Returns the maximum allowed size for an attachment in human format (ie 1Mb)
	 *
	 * @return  string
	 */
	public static function getUploadLimits()
	{
		$mediaLimit = ComponentHelper::getParams('com_media')->get('upload_maxsize', 0) * 1024 * 1024;

		$umf = Format::toBytes(ini_get('upload_max_filesize'));
		$pms = Format::toBytes(ini_get('post_max_size'));

		if (($umf * $pms) == 0)
		{
			$uploadLimit = max($umf, $pms);
		}
		else
		{
			$uploadLimit = min($umf, $pms);
		}

		$uploadLimit = min($uploadLimit, $mediaLimit);
		$uploadLimit = Format::bytesToSize($uploadLimit);

		return $uploadLimit;
	}

	/**
	 * Returns the allowed file extensions for uploads
	 *
	 * @return array
	 */
	public static function allowedExtensions()
	{
		$extensions = ComponentHelper::getParams('com_media')->get('upload_extensions');
		$extensions = explode(',', $extensions);

		// Put everything lowercase...
		$extensions = array_map('strtolower', $extensions);

		// ... and return unique values
		return array_unique($extensions);
	}

	/**
	 * In Joomla 3.5 the modal window to select a Joomla user changed. Since we have to support
	 * versions before 3.5, too, we have to update our code accordingly
	 *
	 * @param   string  $input_id  ID of the input field
	 * @param   string  $name      Name of the input field
	 * @param   string  $value     Value of the field
	 * @param   string  $class     Any additional class for the field
	 *
	 * @return string
	 */
	public static function modalChooseUser($input_id, $name, $value = null, $class = null)
	{
		$container = Container::getInstance('com_ats');
		$container->template->addJS(JUri::root() . 'media/jui/js/fielduser.js');
		$selectText = Text::_('JLIB_FORM_CHANGE_USER');

		// Load the current username if available.
		$table = JTable::getInstance('user');
		if ($value)
		{
			$table->load($value);
		}
		else
		{
			$table->name = Text::_('JLIB_FORM_SELECT_USER');
		}

		$html = <<<HTML
<div class="field-user-wrapper"
	data-url="index.php?option=com_users&amp;view=users&amp;layout=modal&amp;tmpl=component&amp;required=0&amp;field={$input_id}&amp;"
	data-modal=".modal"
	data-modal-width="100%"
	data-modal-height="400px"
	data-input=".field-user-input"
	data-input-name=".field-user-input-name"
	data-button-select=".button-select"
>
	<div class="input-append">
		<input type="text" id="{$input_id}_name" value="{$table->name}" readonly class="field-user-input-name " />
		<a class="btn btn-primary button-select" title="{$selectText}"><span class="icon-user"></span></a>
		<div id="userModal_jform_created_by" tabindex="-1" class="modal hide fade">
			<div class="modal-header">
				<button type="button" class="close" data-dismiss="modal">×</button>
					<h3>{$selectText}</h3>
			</div>
			
			<div class="modal-body"></div>
			<div class="modal-footer">
				<button class="btn" data-dismiss="modal">Cancel</button></div>
			</div>
		</div>
		
		<input type="hidden" id="{$input_id}_id" name="$name" value="$value" class="field-user-input " />
</div>

HTML;

		return $html;
	}

	public static function ordering(Raw $view, $orderingField, $orderingValue)
	{
		$ordering = $view->getLists()->order == $orderingField;
		$class    = 'input-mini';
		$icon     = 'icon-menu';

		// Default inactive ordering
		$html = '<span class="sortable-handler inactive" >';
		$html .= '<span class="' . $icon . '"></span>';
		$html .= '</span>';

		// The modern drag'n'drop method
		if ($view->getPerms()->editstate)
		{
			$disableClassName = '';
			$disabledLabel    = '';

			// DO NOT REMOVE! It will initialize Joomla libraries and javascript functions
			$hasAjaxOrderingSupport = $view->hasAjaxOrderingSupport();

			if (!$hasAjaxOrderingSupport['saveOrder'])
			{
				$disabledLabel    = Text::_('JORDERINGDISABLED');
				$disableClassName = 'inactive tip-top';
			}

			$orderClass = $ordering ? 'order-enabled' : 'order-disabled';

			$html = '<div class="' . $orderClass . '">';
			$html .= '<span class="sortable-handler ' . $disableClassName . '" title="' . $disabledLabel . '" rel="tooltip">';
			$html .= '<span class="' . $icon . '"></span>';
			$html .= '</span>';

			if ($ordering)
			{
				$html .= '<input type="text" name="order[]" style="display: none" size="5" class="' . $class . ' text-area-order" value="' . $orderingValue . '" />';
			}

			$html .= '</div>';
		}

		return $html;
	}

	/**
	 * Decodes the type of a custom field
	 *
	 * @param   string  $selected
	 *
	 * @return  string  The type of the custom field
	 */
	public static function getFieldType($selected)
	{
		static $fieldTypes = [];

		if (!$fieldTypes)
		{
			$basepath = JPATH_ADMINISTRATOR . '/components/com_ats/CustomField';

			$files = JFolder::files($basepath, '.php');

			foreach ($files as $file)
			{
				if ($file === 'Base.php')
				{
					continue;
				}

				$type      = basename($file, '.php');
				$className = 'Akeeba\\TicketSystem\\Admin\\CustomField\\' . $type;

				if (class_exists($className))
				{
					$fieldTypes[strtolower($type)] = Text::_('COM_ATS_CUSTOMFIELDS_FIELD_TYPE_' . strtoupper($type));
				}
			}
		}

		if (!isset($fieldTypes[$selected]))
		{
			return '';
		}

		return $fieldTypes[$selected];
	}
}
